@@ -6,7 +6,7 @@
 #ifndef _IP_VS_H
 #define _IP_VS_H
 
-#define IP_VS_VERSION_CODE            0x000207
+#define IP_VS_VERSION_CODE            0x000208
 #define NVERSION(version)                       \
         (version >> 16) & 0xFF,                 \
         (version >> 8) & 0xFF,                  \
@@ -420,7 +420,6 @@ extern unsigned int check_for_ip_vs_out(struct sk_buff **skb_p,
 
 extern struct ip_vs_timeout_table vs_timeout_table;
 extern struct ip_vs_timeout_table vs_timeout_table_dos;
-extern atomic_t ip_vs_conn_no_cport_cnt;
 
 extern int ip_vs_conn_hash(struct ip_vs_conn *cp);
 extern int ip_vs_conn_unhash(struct ip_vs_conn *cp);
@@ -428,6 +427,7 @@ extern struct ip_vs_conn *ip_vs_conn_in_get
 (int protocol, __u32 s_addr, __u16 s_port, __u32 d_addr, __u16 d_port);
 extern struct ip_vs_conn *ip_vs_conn_out_get
 (int protocol, __u32 s_addr, __u16 s_port, __u32 d_addr, __u16 d_port);
+extern void __ip_vs_conn_put(struct ip_vs_conn *cp);
 extern void ip_vs_conn_put(struct ip_vs_conn *cp);
 extern struct ip_vs_conn *
 ip_vs_conn_new(int proto, __u32 maddr, __u16 mport, __u32 saddr,
@@ -445,11 +445,6 @@ extern void ip_vs_random_dropentry(void);
 extern int ip_vs_conn_init(void);
 extern void ip_vs_conn_cleanup(void);
 
-static inline void __ip_vs_conn_put(struct ip_vs_conn *cp) 
-{
-	atomic_dec(&cp->refcnt);
-}
-
 static inline void ip_vs_control_del(struct ip_vs_conn *cp)
 {
 	struct ip_vs_conn *ctl_cp = cp->control;
@@ -498,7 +493,6 @@ ip_vs_control_add(struct ip_vs_conn *cp, struct ip_vs_conn *ctl_cp)
 }
 
 
-
 /*
  *      IPVS application functions
  *      (from ip_vs_app.c)
