@@ -1,7 +1,7 @@
 /*
  * IPVS         Application module
  *
- * Version:     $Id: ip_vs_app.c,v 1.9 2001/03/22 12:57:46 wensong Exp $
+ * Version:     $Id: ip_vs_app.c,v 1.10 2001/04/01 11:46:26 wensong Exp $
  *
  * Authors:     Wensong Zhang <wensong@linuxvirtualserver.org>
  *
@@ -20,10 +20,6 @@
  *
  */
 
-#ifdef MODULE
-#define EXPORT_SYMTAB
-#endif
-
 #include <linux/config.h>
 #include <linux/module.h>
 #include <linux/types.h>
