@@ -5,7 +5,7 @@
  *              high-performance and highly available server based on a
  *              cluster of servers.
  *
- * Version:     $Id: ip_vs_sched.c,v 1.7 2001/01/08 14:08:53 wensong Exp $
+ * Version:     $Id: ip_vs_sched.c,v 1.8 2001/03/22 12:57:46 wensong Exp $
  *
  * Authors:     Wensong Zhang <wensong@linuxvirtualserver.org>
  *              Peter Kese <peter.kese@ijs.si>
@@ -120,10 +120,10 @@ static struct ip_vs_scheduler *ip_vs_sched_getbyname(const char *sched_name)
 	for (e=l->next; e!=l; e=e->next) {
 		sched = list_entry(e, struct ip_vs_scheduler, n_list);
 		
-		/*
+                /*
                  * Test and MOD_INC_USE_COUNT atomically
                  */
-		if (!try_inc_mod_count(sched->module)) {
+		if (sched->module && !try_inc_mod_count(sched->module)) {
 			/*
                          * This scheduler is just deleted
                          */
@@ -134,7 +134,8 @@ static struct ip_vs_scheduler *ip_vs_sched_getbyname(const char *sched_name)
 			read_unlock_bh(&__ip_vs_sched_lock);
 			return sched;
 		}
-		__MOD_DEC_USE_COUNT(sched->module);
+                if (sched->module)
+                        __MOD_DEC_USE_COUNT(sched->module);
 	}
 
 	read_unlock_bh(&__ip_vs_sched_lock);
@@ -193,7 +194,8 @@ int register_ip_vs_scheduler(struct ip_vs_scheduler *scheduler)
 	 */
 	sched = ip_vs_sched_getbyname(scheduler->name);
         if (sched) {
-                __MOD_DEC_USE_COUNT(sched->module);
+		if (sched->module)
+                        __MOD_DEC_USE_COUNT(sched->module);
 		MOD_DEC_USE_COUNT;
 		IP_VS_ERR("register_ip_vs_scheduler(): [%s] scheduler "
                           "already existed in the system\n", scheduler->name);
